/**
   This file shows how to use Java to write the flatten
   method on lists of lists.

   This file also shows how incredibly difficult it is
   to work with lists in Java. Look at the code to create
   a simple list of lists of integers.
*/

public class Flatten
{
   /**
      Flatten a list of lists of T into a list of T.
   */
   public static <T> List<T> flatten(List<List<T>> head)
   {
      if (null == head)
      {
         return null;
      }
      else
      {
         List<T> first = head.data;
         List<List<T>> rest = head.tail;
         if (null == first)
         {
            return flatten(rest);
         }
         else
         {
            T firstOfFirst = first.data;
            List<T> restOfFirst = first.tail;
            return new List<T>(firstOfFirst, flatten(new List<List<T>>(restOfFirst, rest)));
         }
      }
   }


   public static void main(String[] args)
   {
      // a list of lists of integers, a = [[3, 6, 4], [8, 9], [0, 1, 4], [2, 6, 4, 7], [8, 5]]
      List<List<Integer>> a =
        new List<List<Integer>>(
          new List<Integer>(3, new List<Integer>(6, new List<Integer>(4, null))),
            new List<List<Integer>>(
              new List<Integer>(8, new List<Integer>(9, null)),
                new List<List<Integer>>(
                  new List<Integer>(0, new List<Integer>(1, new List<Integer>(4, null))),
                    new List<List<Integer>>(
                      new List<Integer>(2, new List<Integer>(6, new List<Integer>(4, new List<Integer>(7,null)))),
                        new List<List<Integer>>(
                          new List<Integer>(8, new List<Integer>(5, null)),
                            null)))));

      System.out.println( "a = " + a );
      System.out.println( "flatten a = " + flatten(a) );
   }
}


// A generic List class.
class List<T>
{
   T data;
   List<T> tail; // The tail is either empty (null) or the rest of the List

   List(T data, List<T> next)
   {
      this.data = data;
      this.tail = next;
   }

   public String toString()
   {
      String result = "[" + data;
      List<T> tail = this.tail;
      while (tail != null)
      {
         result += " " + tail.data;
         tail = tail.tail;
      }
      result += "]";
      return result;
   }
}

/*
   https://rosettacode.org/wiki/Flatten_a_list#Java
*/
