/**
   Java versions of some of the functions from recursion.hs.

   Each Java version comes in four flavors. There are iterative
   and recursive versions on an array and then iterative and
   recursive versions on a linked list. The recursive version
   on a linked list is most like the Haskell version.

   Run this code using either the DrJava or jGRASP debuggers in
   order to "see" how the recursion works.

   None of these methods are at all generic. You should rewrite
   them to be generic (like the Haskell functions).
*/

public class Recursion
{
   // sum an array iteratively
   public static int sumAI(int[] a)
   {
      if (null == a) { /* ???? */ }       // what to do for a "null" array?
      int result = 0;
      for (int i = 0; i < a.length; i++)  // works for an empty array
      {
         result += a[i]; // mutate the result variable
      }
      return result;
   }

   // sum an array recursively
   public static int sumAR(int[] a, int low, int hi)
   {
      if (null == a) { /* ???? */ } // what to do for a "null" array?
      if (low > hi)                 // an empty array
         return 0;
      int r = sumAR(a, low+1, hi);  // recursion
      return a[low] + r;
   }

   // sum a linked list iteratively
   public static int sumLI(List head)
   {
      int result = 0;
      while (null != head)
      {
         result += head.data;  // mutate result variable
         head = head.tail;     // mutate list variable
      }
      return result;
   }

   // sum a linked list recursively
   public static int sumLR(List head)
   {
      if (null == head) // empty list
         return 0;
      int r = sumLR(head.tail); // recursion
      return head.data + r;
   }



   // find the max of an array iteratively
   public static int maxAI(int[] a)
   {
      int max = Integer.MIN_VALUE;        // like negative infinity
      for (int i = 0; i < a.length; i++)  // works for an empty array
      {
         if (a[i] >= max)
            max = a[i];    // mutate a local variable
      }
      return max;
   }

   // find the max of an array recursively
   public static int maxAR(int[] a, int low, int hi)
   {
      if (low > hi)                // an empty array
         return Integer.MIN_VALUE; // like negative infinity
      int r = maxAR(a, low+1, hi); // recursion
      if (a[low] >= r)
         return a[low];
      else
         return r;
   }

   // find the max of a linked list iteratively
   public static int maxLI(List head)
   {
      int max = Integer.MIN_VALUE; // like negative infinity
      while (null != head)
      {
         if (head.data >= max)
            max = head.data;   // mutate max variable
         head = head.tail;     // mutate list variable
      }
      return max;
   }

   // find the max of a linked list recursively
   public static int maxLR(List head)
   {
      if (null == head)             // an empty list
         return Integer.MIN_VALUE;  // like negative infinity
      int r = maxLR(head.tail);     // recursion
      if (head.data >= r)
         return head.data;
      else
         return r;
   }



   public static void main(String[] args)
   {
      // an empty array
      int[] a = new int[0];
      // an empty list
      List b = null;

      // a non empty array
      int c[] = {3, 6, 4, 8, 9, 0, 1, 4, 2, 6, 4, 7, 8, 5};
      // a non empty list
      List d = new List(3, new List(6, new List(4, new List(8,
               new List(9, new List(0, new List(1, new List(4,
               new List(2, new List(6, new List(4, new List(7,
               new List(8, new List(5, null))))))))))))));

      // check our methods on the empty array
      System.out.println( "sum = " + sumAI(a) );
      System.out.println( "sum = " + sumAR(a, 0, a.length-1) );
      System.out.println( "max = " + maxAI(a) );
      System.out.println( "max = " + maxAR(a, 0, a.length-1) );

      // check our methods on the empty list
      System.out.println( "sum = " + sumLI(b) );
      System.out.println( "sum = " + sumLR(b) );
      System.out.println( "max = " + maxLI(b) );
      System.out.println( "max = " + maxLR(b) );

      // check our methods on the non empty array
      System.out.println( "sum = " + sumAI(c) );
      System.out.println( "sum = " + sumAR(c, 0, c.length-1) );
      System.out.println( "max = " + maxAI(c) );
      System.out.println( "max = " + maxAR(c, 0, c.length-1) );

      // check our methods on the non empty list
      System.out.println( "sum = " + sumLI(d) );
      System.out.println( "sum = " + sumLR(d) );
      System.out.println( "max = " + maxLI(d) );
      System.out.println( "max = " + maxLR(d) );
   }
}



class List
{
   int data;
   List tail; // The tail is either empty (null) or the rest of the List

   List(int data, List next)
   {
      this.data = data;
      this.tail = next;
   }

   public String toString()
   {
      String result = "[" + data;
      List tail = this.tail;
      while (tail != null)
      {
         result += " " + tail.data;
         tail = tail.tail;
      }
      result += "]";
      return result;
   }
}
