
-- http://www2.math.ou.edu/~dmccullough/teaching/f06-6833/haskell/list_commands.pdf

-- https://hackage.haskell.org/package/base-4.8.2.0/docs/Data-List.html



-- 1.) Selecting parts of lists

-- return the first element of a non-empty list
head' :: [a] -> a
head' (x:_) = x


-- return all but the first element of a non-empty list
tail' :: [a] -> [a]
tail' (_:xs) = xs


-- return the last element of a non-empty list
last' :: [a] -> a
last' [x] = x
last' (x:xs) = last' xs


-- return all but the last element of a non-empty list
init' :: [a] -> [a]
init' [x] = []
init' (x:xs) = x : init' xs


-- return the first n elements of a list
take' :: Int -> [a] -> [a]
take' _ [] = []
take' n _ | n <= 0 = []
take' n (x:xs) = x : take' (n - 1) xs

-- take 5 [10, 15..]


-- drop the first n elements of a list
drop' :: Int -> [a] -> [a]
drop' _ [] = []
drop' n xs | n <= 0 = xs
drop' n (_:xs) = drop' (n - 1) xs

-- drop 2 [[5,2,7], [ ], [0,1], [1..3]]



-- 2.) Getting information about a list

-- return the number of elements in the list
length' :: [a] -> Int
length' [] = 0
length' (_:xs) = 1 + length' xs


-- return the n'th element of a list, where the first element is the zeroth
(!!^) :: [a] -> Int -> a
(!!^) (x:_) 0 = x
(!!^) [] n = error "Index is too large"
(!!^) (_:xs) n = (!!^) xs (n - 1)

-- NOTE: !!' is not a legal operator name, so I used !!^.
-- [1..10] !! 7


-- tell whether the input item is a element of the list
elem' :: Eq a => a -> [a] -> Bool
elem' x [] = False
elem' x (y:ys) | (x == y) = True
               | (x /= y) = elem' x ys

-- elem [ ] [[5,2,7], [ ], [0,1], [1..3]]


-- return the sum of the elements in the list
sum' :: Num a => [a] -> a
sum' [] = 0
sum' (x:xs) = x + (sum' xs)

-- return the product of the elements in the list
product' :: Num a => [a] -> a
product' [] = 1
product' (x:xs) = x * (product' xs)

-- return the anding of the elements in the list
and' :: [Bool] -> Bool
and' [] = True
and' (x:xs) = x && (and' xs)

-- return the oring of the elements in the list
or' :: [Bool] -> Bool
or' [] = False
or' (x:xs) = x || (or' xs)



-- 3.) Combining lists

-- concatenate two lists into one list
(++^) :: [a] -> [a] -> [a]
(++^) [] ys = []
(++^) (x:xs) ys = x : (xs ++^ ys)

-- NOTE: ++' is not a legal operator name, so I used ++^.
-- [5,2,7] ++ [0,1]


-- concatenate two lists into one list
concat' :: [[a]] -> [a]
concat' [] = []
concat' (xs:xss) = xs ++ (concat' xss)

-- concat [[5,2,7], [ ], [0,1], [1..3]]


-- return list of pairs of corresponding elements from two lists
zip' :: [a] -> [b] -> [(a,b)]
zip' [] _ = []
zip' _ [] = []
zip' (x:xs) (y:ys) = (x,y) : (zip' xs ys)

-- zip [1..4] "abcdefgh"


-- reverses the zipping process (convert a list of pairs into a pair of lists)
unzip' :: [(a,b)] -> ([a],[b])
unzip' [] = ([],[])
unzip' ((x,y):ps) = (x:xs, y:ys) where (xs, ys) = unzip' ps

-- unzip [(1,'a'),(2,'b'),(3,'c'),(4,'d')]




-- 4.) Creating and manipulating lists


-- make a list of n copies of one element
replicate' :: Int -> a -> [a]
replicate' 0 x = []
replicate' n x = x : replicate (n-1) x

-- replicate 3 'Z'


-- return list in reverse order
reverse' :: [a] -> [a]
reverse' [] = []
reverse' (x:xs) = (reverse xs) ++ [x]

-- reverse "hello_there"


-- split the list into the first n elements and the rest
splitAt' :: Int -> [a] -> ([a],[a])
splitAt' 0 xs = ([], xs)
splitAt' n [] = ([], [])
splitAt' n (x:xs) = (x:ys, zs) where (ys, zs) = splitAt' (n-1) xs

-- splitAt 4 "abcdefg"


-- (from the List library) remove duplicates from the list
nub' :: Eq a => [a] -> [a]
nub' [] = []
nub' (x:xs) | elem x xs = nub' xs
            | otherwise = x : nub' xs

-- nub' [1,3,1,4,3,3]
